
__title__ = 'openbankingbr'
__description__ = 'Python 3 package to extract public data for Open Banking Brasil.'
__url__ = 'https://github.com/knuppe/openbankingbr'
__version__ = '0.1.4'
__author__ = 'Gustavo J Knuppe'
__author_email__ = 'gustavoknuppe@gmail.com'
__license__ = 'MIT'
__copyright__ = 'Copyright (c) 2021 Gustavo J Knuppe'