
nomeProdutos = {
    'CONTA_DEPOSITO_A_VISTA'                                        : 'Conta corrente',
    'CONTA_POUPANCA'                                                : 'Conta poupança',
    'CONTA_PAGAMENTO_PRE_PAGA'                                      : 'Conta de pagamento pré-paga',
    'DESCONTO_DUPLICATAS'                                           : 'Desconto de duplicatas',
    'DESCONTO_CHEQUES'                                              : 'Desconto de cheques',
    'ANTECIPACAO_FATURA_CARTAO_CREDITO'                             : 'Antecipação de fatura de cartão de crédito',
    'OUTROS_DIREITOS_CREDITORIOS_DESCONTADOS'                       : 'Outros direitos creditórios descontados',
    'OUTROS_TITULOS_DESCONTADOS'                                    : 'Outros títulos descontados',
    'EMPRESTIMO_CREDITO_PESSOAL_CONSIGNADO'                         : 'Crédito pessoal consignado',
    'EMPRESTIMO_CREDITO_PESSOAL_SEM_CONSIGNACAO'                    : 'Crédito pessoal sem consignação',
    'EMPRESTIMO_HOME_EQUITY'                                        : 'Home equity',
    'EMPRESTIMO_MICROCREDITO_PRODUTIVO_ORIENTADO'                   : 'Microcrédito produtivo orientado',
    'EMPRESTIMO_CHEQUE_ESPECIAL'                                    : 'Cheque especial',
    'EMPRESTIMO_CONTA_GARANTIDA'                                    : 'Conta garantida',
    'EMPRESTIMO_CAPITAL_GIRO_PRAZO_VENCIMENTO_ATE_365_DIAS'         : 'Capital de giro com prazo de vencimento até 365 dias',
    'EMPRESTIMO_CAPITAL_GIRO_PRAZO_VENCIMENTO_SUPERIOR_365_DIAS'    : 'Capital de giro com prazo de vencimento superior a 365 dias',
    'EMPRESTIMO_CAPITAL_GIRO_ROTATIVO'                              : 'Capital de giro rotativo',
    'FINANCIAMENTO_AQUISICAO_BENS_VEICULOS_AUTOMOTORES'             : 'Aquisição de bens - Veículos automotores',
    'FINANCIAMENTO_AQUISICAO_BENS_OUTROS_BENS'                      : 'Aquisição de bens - Outros bens',
    'FINANCIAMENTO_MICROCREDITO'                                    : 'Financimento microcrédito',
    'FINANCIAMENTO_RURAL_CUSTEIO'                                   : 'Financimento rural - Custeio',
    'FINANCIAMENTO_RURAL_INVESTIMENTO'                              : 'Financimento rural - Investimento',
    'FINANCIAMENTO_RURAL_COMERCIALIZACAO'                           : 'Financimento rural - Comercialização',
    'FINANCIAMENTO_RURAL_INDUSTRIALIZACAO'                          : 'Financimento rural - Industrialização',
    'FINANCIAMENTO_IMOBILIARIO_SISTEMA_FINANCEIRO_HABITACAO_SFH'    : 'Financimento imobiliário - Sistema Financeiro da Habitação (SFH)',
    'FINANCIAMENTO_IMOBILIARIO_SISTEMA_FINANCEIRO_HABITACAO_SFI'    : 'Financimento imobiliário - Sistema Financeiro da Imobiliário (SFI)',

    # Fora do padrão do bacen -
    
    # BANCO ORIGINAL - Tem vários outros fora do padrão. Não vou tratar o lixo todo...
    'CONTA_PAGAMENTO'                                               : 'Conta salário',
    'EMPRESTIMO_CREDITO_PESSOAL'                                    : 'Crédito pessoal sem consignação',
    'EMPRESTIMO_CREDITO_PESSOAL_NAO_CONSIGNADO'                     : 'Crédito pessoal sem consignação',
    
    # Bradesco
    'FINANCIAMENTO_IMOBILIARIO_SISTEMA_FINANCEIRO_HABITACAO-SFH'    : 'Financimento imobiliário - Sistema Financeiro da Habitação (SFH)',
    'FINANCIAMENTO_IMOBILIARIO_SISTEMA_FINANCEIRO_HABITACAO-SFI'    : 'Financimento imobiliário - Sistema Financeiro da Imobiliário (SFI)',
}

chaveCategorias = {
    "personalAccounts"                                              : "Conta Corrente",
    "businessAccounts"                                              : "Conta Corrente",
    "personalCreditCards"                                           : "Cartão de Crédito",
    "businessCreditCards"                                           : "Cartão de Crédito",
    "personalLoans"                                                 : "Empréstimo",
    "businessLoans"                                                 : "Empréstimo",
    "personalFinancings"                                            : "Financiamento",
    "businessFinancings"                                            : "Financiamento",
    "personalInvoiceFinancings"                                     : "Antecipação de Recebíveis",
    "businessInvoiceFinancings"                                     : "Antecipação de Recebíveis",
    "personalUnarrangedAccountOverdraft"                            : "Adiantamento a Depositante",
    "businessUnarrangedAccountOverdraft"                            : "Adiantamento a Depositante",
}